@@ -66,6 +66,13 @@ pub enum Card {
     CompositeCard(Box<CompositeCard>),
     /// Jump to the function that's on the top of the stack
     DynamicJump,
+    /// Get the given integer row of a table
+    Get,
+    /// Arguments: table, value
+    AppendTable,
+    PopTable,
+    /// Create a Table from the results of the provided cards
+    Array(Vec<Card>),
 }
 
 #[derive(Debug, Clone)]
@@ -131,6 +138,10 @@ impl Card {
             Card::CompositeCard(c) => c.ty.as_str(),
             Card::Function(_) => "Function",
             Card::DynamicJump => "Dynamic Jump",
+            Card::Get => "Get",
+            Card::AppendTable => "Append to Table",
+            Card::PopTable => "Pop from Table",
+            Card::Array(_) => "Array",
         }
     }
 
@@ -145,7 +156,8 @@ impl Card {
             | Card::Repeat { .. }
             | Card::ForEach { .. }
             | Card::Pass
-            | Card::CompositeCard { .. } => None,
+            | Card::CompositeCard { .. }
+            | Card::Array(_) => None,
 
             Card::GetProperty => Some(Instruction::GetProperty),
             Card::SetProperty => Some(Instruction::SetProperty),
@@ -179,6 +191,9 @@ impl Card {
             Card::Return => Some(Instruction::Return),
             Card::Len => Some(Instruction::Len),
             Card::DynamicJump => Some(Instruction::CallLane),
+            Card::Get => Some(Instruction::NthRow),
+            Card::AppendTable => Some(Instruction::AppendTable),
+            Card::PopTable => Some(Instruction::PopTable),
         }
     }
 
@@ -224,7 +239,10 @@ impl Card {
             | Instruction::GotoIfFalse
             | Instruction::ForEach
             | Instruction::FunctionPointer
-            | Instruction::BeginForEach => {}
+            | Instruction::BeginForEach
+            | Instruction::AppendTable
+            | Instruction::PopTable
+            | Instruction::NthRow => {}
         };
     }
 
