@@ -141,27 +141,23 @@ impl<'a, Aux> Vm<'a, Aux> {
         self.runtime_data.value_stack.pop()
     }
 
-    pub fn get_table(&self, value: Value) -> Result<&FieldTable, ExecutionErrorPayload> {
+    pub fn get_table(&self, value: Value) -> Result<&CaoLangTable, ExecutionErrorPayload> {
         let res = match value {
             Value::Object(o) => unsafe { &*o },
             _ => {
                 debug!("Got {:?} instead of object", value);
-                return Err(ExecutionErrorPayload::invalid_argument(
-                    "Input must be an Object".to_string(),
-                ));
+                return Err(ExecutionErrorPayload::invalid_argument("Expected Table"));
             }
         };
         Ok(res)
     }
 
-    pub fn get_table_mut(&self, value: Value) -> Result<&mut FieldTable, ExecutionErrorPayload> {
+    pub fn get_table_mut(&self, value: Value) -> Result<&mut CaoLangTable, ExecutionErrorPayload> {
         let res = match value {
             Value::Object(o) => unsafe { &mut *o },
             _ => {
                 debug!("Got {:?} instead of object", value);
-                return Err(ExecutionErrorPayload::invalid_argument(
-                    "GetProperty input must be an Object".to_string(),
-                ));
+                return Err(ExecutionErrorPayload::invalid_argument("Expected Table"));
             }
         };
         Ok(res)
@@ -169,7 +165,7 @@ impl<'a, Aux> Vm<'a, Aux> {
 
     /// Initializes a new FieldTable in this VM instance
     #[inline]
-    pub fn init_table(&mut self) -> Result<std::ptr::NonNull<FieldTable>, ExecutionErrorPayload> {
+    pub fn init_table(&mut self) -> Result<std::ptr::NonNull<CaoLangTable>, ExecutionErrorPayload> {
         self.runtime_data.init_table()
     }
 
@@ -485,6 +481,62 @@ impl<'a, Aux> Vm<'a, Aux> {
                 Instruction::Len => instr_execution::instr_len(self).map_err(|err| {
                     payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
                 })?,
+                Instruction::NthRow => {
+                    let [i, instance] = self.runtime_data.value_stack.pop_n::<2>();
+                    let table = self.get_table_mut(instance).map_err(|err| {
+                        payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
+                    })?;
+                    let i = i.as_int().ok_or_else(|| {
+                        payload_to_error(
+                            ExecutionErrorPayload::invalid_argument(
+                                "Input must be an integer".to_string(),
+                            ),
+                            instr_ptr,
+                            &self.runtime_data.call_stack,
+                        )
+                    })?;
+                    if i < 0 {
+                        return Err(payload_to_error(
+                            ExecutionErrorPayload::invalid_argument(
+                                "Input must be non-negative".to_string(),
+                            ),
+                            instr_ptr,
+                            &self.runtime_data.call_stack,
+                        ));
+                    }
+                    let key = table.nth_key(i as usize);
+                    let value = table.get(&key).copied().unwrap_or(Value::Nil);
+
+                    self.stack_push(value).map_err(|err| {
+                        payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
+                    })?;
+                    self.stack_push(key).map_err(|err| {
+                        payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
+                    })?;
+                }
+                Instruction::AppendTable => {
+                    let instance = self.stack_pop();
+                    let value = self.stack_pop();
+                    let table = self.get_table_mut(instance).map_err(|err| {
+                        payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
+                    })?;
+                    table.append(value).map_err(|err| {
+                        payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
+                    })?;
+                }
+
+                Instruction::PopTable => {
+                    let instance = self.stack_pop();
+                    let table = self.get_table_mut(instance).map_err(|err| {
+                        payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
+                    })?;
+                    let value = table.pop().map_err(|err| {
+                        payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
+                    })?;
+                    self.stack_push(value).map_err(|err| {
+                        payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
+                    })?;
+                }
             }
         }
 
