@@ -1,9 +1,10 @@
 use std::convert::TryFrom;
 
+use bytemuck::Pod;
 use tracing::debug;
 
 use crate::{
-    bytecode::{decode_str, read_from_bytes, TriviallyEncodable},
+    bytecode::{decode_str, read_from_bytes},
     collections::handle_table::Handle,
     compiled_program::CaoCompiledProgram,
     procedures::ExecutionErrorPayload,
@@ -29,7 +30,7 @@ pub fn read_str<'a>(instr_ptr: &mut usize, program: &'a [u8]) -> Option<&'a str>
 ///
 /// Assumes that the underlying data is safely decodable to the given type
 ///
-pub unsafe fn decode_value<T: TriviallyEncodable>(bytes: &[u8], instr_ptr: &mut usize) -> T {
+pub unsafe fn decode_value<T: Pod>(bytes: &[u8], instr_ptr: &mut usize) -> T {
     let (len, val) = read_from_bytes(&bytes[*instr_ptr..]).expect("Failed to read data");
     *instr_ptr += len;
     val
@@ -82,7 +83,7 @@ pub fn instr_len<T>(vm: &mut Vm<T>) -> ExecutionResult {
     let val = vm.stack_pop();
     let len = match val {
         Value::Nil => 0,
-        Value::Integer(_) | Value::Real(_) => 1,
+        Value::Function { .. } | Value::Integer(_) | Value::Real(_) => 1,
         Value::String(s) => {
             let st = unsafe {
                 s.get_str().ok_or_else(|| {
@@ -115,38 +116,59 @@ pub fn instr_string_literal<T>(
     Ok(())
 }
 
+pub fn push_call_frame(
+    arity: usize,
+    src_ptr: u32,
+    instr_ptr: u32,
+    runtime_data: &mut RuntimeData,
+) -> ExecutionResult {
+    // remember the location after this jump
+    runtime_data
+        .call_stack
+        .last_mut()
+        .expect("Call stack was empty")
+        .dst_instr_ptr = instr_ptr;
+
+    // init the new call frame
+    runtime_data
+        .call_stack
+        .push(CallFrame {
+            src_instr_ptr: src_ptr,
+            dst_instr_ptr: instr_ptr,
+            stack_offset: runtime_data
+                .value_stack
+                .len()
+                .checked_sub(arity as usize)
+                .ok_or(ExecutionErrorPayload::MissingArgument)? as u32,
+        })
+        .map_err(|_| ExecutionErrorPayload::CallStackOverflow)?;
+    Ok(())
+}
+
 pub fn instr_jump(
     src_ptr: usize,
     instr_ptr: &mut usize,
     program: &CaoCompiledProgram,
     runtime_data: &mut RuntimeData,
 ) -> ExecutionResult {
-    let label: Handle = unsafe { decode_value(&program.bytecode, instr_ptr) };
-    let argcount: u32 = unsafe { decode_value(&program.bytecode, instr_ptr) };
+    let Value::Function { hash: label, arity } = runtime_data.value_stack.pop() else{
+        return Err(ExecutionErrorPayload::invalid_argument("Jump instruction expects a function argument"));
+    };
 
-    // remember the location after this jump
-    runtime_data
-        .call_stack
-        .last_mut()
-        .expect("Call stack was empty")
-        .dst_instr_ptr = *instr_ptr as u32;
-
-    // init the new call frame
-    runtime_data
-        .call_stack
-        .push(CallFrame {
-            src_instr_ptr: src_ptr as u32,
-            dst_instr_ptr: *instr_ptr as u32,
-            stack_offset: runtime_data
-                .value_stack
-                .len()
-                .checked_sub(argcount as usize)
-                .ok_or(ExecutionErrorPayload::MissingArgument)? as u32,
-        })
-        .map_err(|_| ExecutionErrorPayload::CallStackOverflow)?;
+    push_call_frame(
+        arity as usize,
+        src_ptr as u32,
+        *instr_ptr as u32,
+        runtime_data,
+    )?;
 
     // set the instr_ptr to the new lane's beginning
-    *instr_ptr = program.labels.0.get(label).expect("Label not found").pos as usize;
+    *instr_ptr = program
+        .labels
+        .0
+        .get(label)
+        .ok_or_else(|| ExecutionErrorPayload::ProcedureNotFound(label))?
+        .pos as usize;
     Ok(())
 }
 
