@@ -59,7 +59,7 @@ fn test_trace_entry() {
     let trace = err.trace;
 
     let error_card = ir
-        .get_card(&trace[0])
+        .get_card(&trace[0].index)
         .expect("Expected to find the errored card");
 
     assert!(matches!(error_card, Card::CallNative(_)));
@@ -587,19 +587,19 @@ fn len_test_happy() {
                 .with_card(Card::CreateTable)
                 .with_card(Card::set_var(t))
                 // first property
+                .with_card(Card::ScalarInt(42))
                 .with_card(Card::read_var(t))
                 .with_card(Card::StringLiteral("asd".to_string()))
-                .with_card(Card::ScalarInt(42))
                 .with_card(Card::SetProperty)
                 // same property as above
+                .with_card(Card::ScalarInt(69))
                 .with_card(Card::read_var(t))
                 .with_card(Card::StringLiteral("asd".to_string()))
-                .with_card(Card::ScalarInt(69))
                 .with_card(Card::SetProperty)
                 // new property
+                .with_card(Card::ScalarInt(89))
                 .with_card(Card::read_var(t))
                 .with_card(Card::StringLiteral("basdasd".to_string()))
-                .with_card(Card::ScalarInt(89))
                 .with_card(Card::SetProperty)
                 // len
                 .with_card(Card::read_var(t))
@@ -1037,3 +1037,177 @@ fn set_var_to_empty_test() {
     let mut vm = Vm::new(()).unwrap();
     vm.run(&program).expect("run");
 }
+
+#[test]
+fn callback_test() {
+    let cu = Module {
+        imports: [].into(),
+        submodules: [].into(),
+        lanes: [
+            (
+                "main".to_string(),
+                Lane::default().with_cards(vec![
+                    Card::ScalarInt(0),
+                    Card::set_global_var("i"),
+                    Card::function_value("callback"),
+                    Card::jump("call_callback"),
+                    Card::function_value("callback"),
+                    Card::jump("call_callback"),
+                    Card::function_value("callback"),
+                    Card::jump("call_callback"),
+                    Card::function_value("callback"),
+                    Card::jump("call_callback"),
+                ]),
+            ),
+            (
+                "call_callback".to_string(),
+                Lane::default()
+                    .with_arg("cb")
+                    .with_cards(vec![Card::read_var("cb"), Card::DynamicJump]),
+            ),
+            (
+                "callback".to_string(),
+                Lane::default().with_cards(vec![
+                    Card::read_var("i"),
+                    Card::ScalarInt(1),
+                    Card::Add,
+                    Card::set_global_var("i"),
+                ]),
+            ),
+        ]
+        .into(),
+    };
+
+    let program = compile(cu, CompileOptions::new()).expect("compile");
+
+    let mut vm = Vm::new(()).unwrap();
+    vm.run(&program).expect("run");
+
+    let result = vm
+        .read_var_by_name("i", &program.variables)
+        .expect("Failed to read pooh variable");
+
+    assert_eq!(result.as_int().unwrap(), 4);
+}
+
+#[test]
+fn read_set_property_shorthand_test() {
+    let cu = Module {
+        imports: [].into(),
+        submodules: [].into(),
+        lanes: [(
+            "main".to_string(),
+            Lane::default().with_cards(vec![
+                Card::CreateTable,
+                Card::set_global_var("winnie"),
+                //
+                Card::ScalarInt(1),
+                Card::set_var("winnie.foo"),
+                //
+                Card::ScalarInt(1),
+                Card::read_var("winnie.foo"),
+                Card::Add,
+                Card::set_var("winnie.foo"),
+            ]),
+        )]
+        .into(),
+    };
+
+    let program = compile(cu, CompileOptions::new()).expect("compile");
+
+    let mut vm = Vm::new(()).unwrap();
+    vm.run(&program).expect("run");
+
+    let result = vm
+        .read_var_by_name("winnie", &program.variables)
+        .expect("Failed to read winnie variable");
+
+    let result = unsafe { result.as_table().unwrap() };
+
+    let foo = result.get("foo").unwrap();
+    assert_eq!(*foo, Value::Integer(2));
+}
+
+#[test]
+fn read_property_shorthand_test() {
+    let cu = Module {
+        imports: [].into(),
+        submodules: [].into(),
+        lanes: [(
+            "main".to_string(),
+            Lane::default().with_cards(vec![
+                // table is pushed onto the stack
+                Card::set_global_var("i"),
+                Card::read_var("i.foo"),
+                Card::set_global_var("j"),
+            ]),
+        )]
+        .into(),
+    };
+
+    let program = compile(cu, CompileOptions::new()).expect("compile");
+
+    let mut vm = Vm::new(()).unwrap();
+    unsafe {
+        let mut table_ptr = vm.init_table().unwrap();
+        let table = table_ptr.as_mut();
+        let key = vm.init_string("foo").unwrap();
+        table
+            .insert(Value::String(key), Value::Integer(42))
+            .unwrap();
+        vm.stack_push(Value::Object(table_ptr.as_ptr())).unwrap();
+    }
+
+    vm.run(&program).expect("run");
+
+    let result = vm
+        .read_var_by_name("j", &program.variables)
+        .expect("Failed to read i variable");
+
+    assert_eq!(result, Value::Integer(42));
+}
+
+#[test]
+fn nested_read_set_property_shorthand_test() {
+    let cu = Module {
+        imports: [].into(),
+        submodules: [].into(),
+        lanes: [(
+            "main".to_string(),
+            Lane::default().with_cards(vec![
+                Card::CreateTable,
+                Card::set_global_var("winnie"),
+                //
+                Card::CreateTable,
+                Card::set_var("winnie.foo"),
+                Card::CreateTable,
+                Card::set_var("winnie.foo.bar"),
+                Card::CreateTable,
+                Card::set_var("winnie.foo.bar.baz"),
+                //
+                Card::ScalarInt(42),
+                Card::set_var("winnie.foo.bar.baz.pooh"),
+            ]),
+        )]
+        .into(),
+    };
+
+    let program = compile(cu, CompileOptions::new()).expect("compile");
+
+    let mut vm = Vm::new(()).unwrap();
+    vm.run(&program).expect("run");
+
+    let result = vm
+        .read_var_by_name("winnie", &program.variables)
+        .expect("Failed to read winnie variable");
+
+    unsafe {
+        let result = result.as_table().unwrap();
+        let foo = result.get("foo").unwrap().as_table().unwrap();
+        let bar = foo.get("bar").unwrap().as_table().unwrap();
+        let baz = bar.get("baz").unwrap().as_table().unwrap();
+        let pooh = baz.get("pooh").unwrap().as_int().unwrap();
+
+        assert_eq!(pooh, 42);
+    }
+}
